-- D42 packer V1.1

strUsage = [[
Usage: lua d42_exomize.lua <input_dir> <temp_dir> <output_dir>

input_dir  = directory where the original game files are located
temp_dir   = work directory for the unpacked room files
output_dir = destination directory for the compressed files 
The directories must exist.
]]

strExoPath = "exomizer.exe"

function printf (...) print(string.format(...)) end

function exomize_file(strInPath, strOutPath)
	local strCmd = string.format("%s mem -o %s %s", strExoPath, strOutPath, strInPath)
	local iRet = os.execute(strCmd)
	assert(iRet == 0, "error calling exomizer")
end

function depackD42(bin)
	local pos = 0
	local function getbyte() 
		pos = pos + 1
		return bin:byte(pos)
	end
	
	local unpacked = {}
	local function putbyte(b)
		table.insert(unpacked, string.char(b))
	end
	
	putbyte(getbyte()) -- copy load addr
	putbyte(getbyte())
	
	local packbyte = getbyte()
	
	while (pos<bin:len()) do
		local b = getbyte()
		if b == packbyte then
			local count = getbyte()
			b = getbyte()
			if count== 0 then break end
			for i=1,count do putbyte(b) end
		else
			putbyte(b)
		end
	end

	if #unpacked < 0x1f02 then
		printf("unpacked room file is too short: 0x%x", #unpacked)
	elseif #unpacked > 0x2002 then
		printf("unpacked room file exceeds area: 0x%x", #unpacked)
	end
		
	if pos < bin:len() then
		printf("depackD4: End marker before EOF: pos = 0x%x, packed len = 0x%x, unpacked len = 0x%x",
		pos, bin:len(), #unpacked)
	end
	
	local unpacked_bin = table.concat(unpacked)
	local unpacked_len = unpacked_bin:len()
	local default_len = 0x1f02
	
	if unpacked_len < default_len then	
		printf("depackD4: unpacked file is too short ($%04x bytes)", unpacked_len)
		error()
	elseif unpacked_len > default_len then
		printf("depackD4: unpacked file is too long ($%04x), truncating ", unpacked_len)
		unpacked_bin = unpacked_bin:sub(1, default_len)
	end
	return unpacked_bin
end

function try_load_file(strPath)
	printf("Probing file %s", strPath)
	local fd, msg = io.open(strPath, "rb")
	if fd then
		print("Found, loading")
		local strBin = fd:read("*a")
		fd:close()
		return strBin
	else
		printf("File not found: %s", msg)
	end
end

function write_file(strPath, strBin)
	printf("Writing file %s", strPath)
	local fd, msg = io.open(strPath, "wb")
	assert(fd, msg)
	fd:write(strBin)
	fd:close()
end

if #arg ~= 3 then 
	print(strUsage) 
	os.exit(0)
else
	strInDir = arg[1]
	strTmpDir = arg[2]
	strOutDir = arg[3]
	
	-- base files ba, oj, o2, nw
	astrBaseFiles = {"ba", "oj", "o2", "nw"}
	for i, strName in ipairs(astrBaseFiles) do
		local strInPath = string.format("%s/%s", strInDir, strName)
		local strOutPath = string.format("%s/%s", strOutDir, strName)
		exomize_file(strInPath, strOutPath)
	end
	
	-- rooms 00-7f
	for i = 0, 127 do
		local strInPath = string.format("%s/%02x", strInDir, i)
		local strTmpPath = string.format("%s/%02x", strTmpDir, i)
		local strOutPath = string.format("%s/%02x", strOutDir, i)
		local strBin = try_load_file(strInPath)
		if strBin then
			local strUnpackedBin = depackD42(strBin)
			print(strTmpPath)
			write_file(strTmpPath, strUnpackedBin)
			exomize_file(strTmpPath, strOutPath)
		else
		
		end
	end
	
	-- musics m0-mf
	for i = 0, 15 do
		local strInPath = string.format("%s/m%1x", strInDir, i)
		local strOutPath = string.format("%s/m%1x", strOutDir, i)
		local strBin = try_load_file(strInPath)
		if strBin then
			exomize_file(strInPath, strOutPath)
		end
	end
	
end

